@@ -1,3 +1,5 @@
+from StringIO import StringIO
+
 from webob import *
 
 def simple_app(environ, start_response):
@@ -27,12 +29,20 @@ def test_response():
     assert res.headers['content-type'] == 'text/html'
     assert res.headerlist == [('content-type', 'text/html')]
     res.set_cookie('x', 'y')
-    assert res.headers['set-cookie'] == 'x=y; Path=/;'
-    res = Response('200 OK', content_type='text/html',
-                   body='a body')
+    assert res.headers['set-cookie'].strip(';') == 'x=y; Path=/'
+    res = Response('a body', '200 OK', content_type='text/html')
     res.encode_content()
     assert res.content_encoding == 'gzip'
     assert res.body == '\x1f\x8b\x08\x00\x00\x00\x00\x00\x02\xffKTH\xcaO\xa9\x04\x00\xf6\x86GI\x06\x00\x00\x00'
     res.decode_content()
     assert res.content_encoding is None
     assert res.body == 'a body'
+
+def test_HEAD_closes():
+    req = Request.blank('/')
+    req.method = 'HEAD'
+    app_iter = StringIO('foo')
+    res = req.get_response(Response(app_iter=app_iter))
+    assert res.status_int == 200
+    assert res.body == ''
+    assert app_iter.closed
