@@ -56,6 +56,8 @@ def html_escape(s):
     """
     if s is None:
         return ''
+    if hasattr(s, '__html__'):
+        return s.__html__()
     if not isinstance(s, basestring):
         if hasattr(s, '__unicode__'):
             s = unicode(s)
@@ -184,6 +186,34 @@ class header_getter(object):
     def __repr__(self):
         return '<Proxy for header %s>' % self.header
 
+class set_via_call(object):
+    def __init__(self, func, adapt_args=None):
+        self.func = func
+        self.adapt_args = adapt_args
+    def __get__(self, obj, type=None):
+        return self.__class__(self.func.__get__(obj, type))
+    def __set__(self, obj, value):
+        if self.adapt_args is None:
+            args, kw = (value,), {}
+        else:
+            result = self.adapt_args(value)
+            if result is None:
+                return
+            args, kw = result
+        self.func(obj, *args, **kw)
+    def __repr__(self):
+        return 'set_via_call(%r)' % self.func
+    def __call__(self, *args, **kw):
+        return self.func(*args, **kw)
+
+def _adapt_cache_expires(value):
+    if value is False:
+        return None
+    if value is True:
+        return (0,), {}
+    else:
+        return (value,), {}
+
 class converter(object):
     """
     Wraps a decorator, and applies conversion for that decorator
@@ -506,7 +536,7 @@ class Request(object):
         for name, value in kw.items():
             if not hasattr(self.__class__, name):
                 raise TypeError(
-                    "Unexpected keyword: %s=%r" % name, value)
+                    "Unexpected keyword: %s=%r" % (name, value))
             setattr(self, name, value)
 
     def __setattr__(self, attr, value, DEFAULT=[]):
@@ -1087,7 +1117,7 @@ class Request(object):
             ## FIXME: Should we use .tell() to try to put the body
             ## back to its previous position?
             input.seek(0)
-        if length == -1:
+        if length in (-1, None):
             body = self.body
             length = len(body)
             self.content_length = length
@@ -1233,7 +1263,7 @@ class Request(object):
     user_agent = environ_getter('HTTP_USER_AGENT', rfc_section='14.43')
 
     def __repr__(self):
-        msg = '<%s at %x %s %s>' % (
+        msg = '<%s at 0x%x %s %s>' % (
             self.__class__.__name__,
             abs(id(self)), self.method, self.url)
         return msg
@@ -1407,6 +1437,7 @@ class Response(object):
 
     default_content_type = 'text/html'
     default_charset = 'UTF-8'
+    unicode_errors = 'strict'
     default_conditional_response = False
 
     def __init__(self, body=None, status='200 OK', headerlist=None, app_iter=None,
@@ -1468,7 +1499,7 @@ class Response(object):
             setattr(self, name, value)
 
     def __repr__(self):
-        return '<%s %x %s>' % (
+        return '<%s at 0x%x %s>' % (
             self.__class__.__name__,
             abs(id(self)),
             self.status)
@@ -1720,7 +1751,7 @@ class Response(object):
             raise AttributeError(
                 "You cannot access Response.unicode_body unless charset is set")
         body = self.body
-        return body.decode(self.charset)
+        return body.decode(self.charset, self.unicode_errors)
 
     def _unicode_body__set(self, value):
         if not self.charset:
@@ -1863,6 +1894,11 @@ class Response(object):
         return location
 
     def _location__set(self, value):
+        if value is None:
+            del self.location
+            return
+        if isinstance(value, unicode):
+            value = value.encode('ISO-8859-1')
         if not _SCHEME_RE.search(value):
             # Not absolute, see if we can make it absolute
             if self.request is not None:
@@ -1933,7 +1969,7 @@ class Response(object):
 
     cache_control = property(_cache_control__get, _cache_control__set, _cache_control__del, doc=_cache_control__get.__doc__)
 
-    def cache_expires(self, seconds=0, **kw):
+    def _cache_expires(self, seconds=0, **kw):
         """
         Set expiration on this request.  This sets the response to
         expire in the given seconds, and any other attributes are used
@@ -1963,6 +1999,8 @@ class Response(object):
         for name, value in kw.items():
             setattr(cache_control, name, value)
 
+    cache_expires = set_via_call(_cache_expires, _adapt_cache_expires)
+
     content_encoding = header_getter('Content-Encoding', rfc_section='14.11')
 
     def encode_content(self, encoding='gzip'):
@@ -2041,8 +2079,11 @@ class Response(object):
         """
         if body is None:
             body = self.body
-        import md5
-        h = md5.new(body)
+        try:
+            from hashlib import md5
+        except ImportError:
+            from md5 import md5
+        h = md5(body)
         md5_digest = h.digest().encode('base64').replace('\n', '').strip('=')
         self.etag = md5_digest
         if set_content_md5:
@@ -2127,7 +2168,7 @@ class Response(object):
         start_response(self.status, self.headerlist)
         if environ['REQUEST_METHOD'] == 'HEAD':
             # Special case here...
-            return []
+            return EmptyResponse(self.app_iter)
         return self.app_iter
 
     _safe_methods = ('GET', 'HEAD')
@@ -2154,10 +2195,10 @@ class Response(object):
                     status304 = False
         if status304:
             start_response('304 Not Modified', self.headerlist)
-            return []
+            return EmptyResponse(self.app_iter)
         if req.method == 'HEAD':
             start_response(self.status, self.headerlist)
-            return []
+            return EmptyResponse(self.app_iter)
         if (req.range and req.if_range.match_response(self)
             and self.content_range is None
             and req.method == 'GET'
@@ -2258,7 +2299,7 @@ class FakeCGIBody(object):
         inner = repr(self.vars)
         if len(inner) > 20:
             inner = inner[:15] + '...' + inner[-5:]
-        return '<%s at %x viewing %s>' % (
+        return '<%s at 0x%x viewing %s>' % (
             self.__class__.__name__,
             abs(id(self)), inner)
 
@@ -2377,3 +2418,23 @@ class AppIterRange(object):
         self._served += len(chunk)
         return chunk
 
+class EmptyResponse(object):
+
+    """An empty WSGI response.
+
+    An iterator that immediately stops. Optionally provides a close
+    method to close an underlying app_iter it replaces.
+    """
+
+    def __init__(self, app_iter=None):
+        if app_iter and hasattr(app_iter, 'close'):
+            self.close = app_iter.close
+
+    def __iter__(self):
+        return self
+
+    def __len__(self):
+        return 0
+
+    def next(self):
+        raise StopIteration()
